@@ -4,7 +4,8 @@ Module providing functionality to connect to a running Rserve instance
 """
 import socket
 import time
-###
+import pydoc
+
 from . import rtypes
 from .rexceptions import RConnectionRefused, REvalError, PyRserveClosed
 from .rserializer import rEval, rAssign, rSerializeResponse, rShutdown
@@ -15,7 +16,7 @@ RSERVEPORT = 6311
 DEBUG = False
 
 
-def _defaultOOBCallback(data, code=0):
+def _defaultOOBCallback(data, code=0):  # noqa
     return None
 
 
@@ -73,7 +74,16 @@ def checkIfClosed(func):
     def decoCheckIfClosed(self, *args, **kw):
         if self.isClosed:
             raise PyRserveClosed('Connection to Rserve already closed')
-        return func(self, *args, **kw)
+        try:
+            return func(self, *args, **kw)
+        except socket.error as msg:
+            if msg.strerror in ['Connection reset by peer', 'Broken pipe']:
+                # seems like the connection to Rserve has died, so mark
+                # the connection as closed
+                self.close()
+                raise PyRserveClosed('Connection to Rserve already closed')
+            else:
+                raise
     return decoCheckIfClosed
 
 
@@ -125,6 +135,7 @@ class RConnector(object):
         self.sock.close()
         self.__closed = True
 
+    @checkIfClosed
     def shutdown(self):
         rShutdown(fp=self.sock)
         self.close()
@@ -290,7 +301,7 @@ class RNameSpace(object):
         realname = name[1:] if name.startswith('_') else name
         try:
             isFunction = self._rconn.isFunction(realname)
-        except:
+        except Exception:
             # an error is only raised if neither such a function or variable
             # exists at all!
             raise NameError('no such variable or function "%s" '
@@ -319,7 +330,7 @@ class RNameSpaceReference(object):
         """Return either a reference proxy to a variable to to a function"""
         try:
             isFunction = self._rconn.isFunction(name)
-        except:
+        except Exception:
             # an error is only raised if neither such a function or variable
             # exists at all!
             raise NameError('no such variable or function "%s" '
@@ -357,21 +368,44 @@ class RFuncProxy(RBaseProxy):
     def __call__(self, *args, **kw):
         return self._rconn.callFunc(self.__name__, *args, **kw)
 
+    # command to send to R in order to get the help for a function in text
+    # format:
+    R_HELP = "capture.output(tools:::Rd2txt(utils:::.getHelpFile(help(%s))))"
+
     @property
     def __doc__(self):
+        """
+        There are different ways to get the help message from R:
+        # The the package db file:
+        pkgRdDB = tools:::fetchRdDB(file.path(find.package('base'),
+                                              'help', 'base'))
+        # show all available topics in the help package:
+        names(pkgRdDB)
+        # convert the 'lapply' help message to text (from the base package):
+        tools::Rd2txt(pkgRdDB[['lapply']])
+        # capture this output into a variable:
+        a <- capture.output(tools::Rd2txt(pkgRdDB[['lapply']]))
+        Disadvantage: One needs to know the package beforehand.
+
+        Better:
+        Everything in one line and better (doesn't need to know the pkg):
+        a <- capture.output(tools:::Rd2txt(utils:::.getHelpFile(help(sapply))))
+        """
         try:
-            d = self._rconn.eval('readLines(as.character(help(%s)))' %
-                                 self.__name__)
+            d = self._rconn.eval(self.R_HELP % self.__name__)
         except REvalError:
             # probably no help available, unfortunately there is no specific
             # code for this...
             return None
+        # Join the list of strings:
         helpstring = '\n'.join(d)
-        helpstring = helpstring.replace('_\x08', '')
+        # remove some obscure characters:
+        # helpstring = helpstring.replace('_\x08', '')
         return helpstring
 
     def help(self):
-        print(self.__doc__)
+        """Directly page the help message to the terminal (e.g. via less)"""
+        pydoc.pager(self.__doc__)
 
     def __getattr__(self, name):
         """Allow for nested name space calls, e.g. 't.test' """
@@ -383,11 +417,11 @@ class RFuncProxy(RBaseProxy):
         concatName = "%s.%s" % (self.__name__, name)
         try:
             self._rconn.isFunction(concatName)
-        except:
+        except Exception:
             # an error is only raised if neither such a function or variable
             # exists at all!
             raise NameError('no such variable or function "%s" '
-                            'defined in Rserve' % concatName)
+                            'defined in R' % concatName)
         return RFuncProxy(concatName, self._rconn)
 
 
@@ -406,7 +440,7 @@ def _test_main():
     conn = connect()
     print('"conn" is your handle to rserve. Type e.g. "conn(\'1\')" '
           'for string evaluation.')
-    #r('x<-1:20; y<-x*2; lm(y~x)')
+    # r('x<-1:20; y<-x*2; lm(y~x)')
     sc = open('../testData/test-script.R').read()
     v = conn.r(sc)
     open('r-test-png.png', 'w').write(v[3])
