@@ -2,13 +2,23 @@
 """
 Parser module for pyRserve
 """
+import io
 import struct
 import socket
-import io
-###
-from .rtypes import *
+
+import numpy
+
+from .rtypes import (
+    CMD_OOB, CMD_RESP, DT_SEXP, DTs, ERRORS, RESP_ERR, RESP_OK,
+    SOCKET_BLOCK_SIZE, VALID_R_TYPES, XT_ARRAY_BOOL, XT_ARRAY_CPLX,
+    XT_ARRAY_DOUBLE, XT_ARRAY_INT, XT_ARRAY_STR, XT_BOOL, XT_CLOS, XT_DOUBLE,
+    XT_HAS_ATTR, XT_INT, XT_INT3, XT_INT7, XT_LANG_NOTAG, XT_LANG_TAG, XT_LARGE,
+    XT_LIST_NOTAG, XT_LIST_TAG, XT_NULL, XT_RAW, XT_S4, XT_STR, XT_SYMNAME,
+    XT_UNKNOWN, XT_VECTOR, XT_VECTOR_EXP, XTs, structMap, numpyMap
+)
 from .misc import FunctionMapper, byteEncode, stringEncode, PY3
-from .rexceptions import RResponseError, REvalError
+from .rexceptions import \
+    RResponseError, REvalError, EndOfDataError, RParserError
 from .taggedContainers import TaggedList, asTaggedArray, asAttrArray
 
 DEBUG = 0
@@ -29,7 +39,7 @@ class OOBMessage(object):
         self.messageSize = messageSize
 
     def __len__(self):
-        return self.messageSize + 16 #header
+        return self.messageSize + 16  # header
 
 
 class Command(object):
@@ -38,7 +48,7 @@ class Command(object):
     """
     def __init__(self, code):
         self.code = code
-        #Rserve 1.7 or’s the command with CMD_RESP even if it’s a OOB instead
+        # Rserve 1.7 or’s the command with CMD_RESP even if it’s a OOB instead
         fixedOOBCode = code & ~CMD_RESP
 
         self.isOOB = bool(code & CMD_OOB)
@@ -46,7 +56,8 @@ class Command(object):
         self.oobUserCode = fixedOOBCode & 0xfff
 
         self.errCode = (code >> 24) & 127
-        self.responseCode = code & 0xfffff #lowest 20 bit
+        self.responseCode = code & 0xfffff  # lowest 20 bit
+
 
 class Lexeme(list):
     """Basic Lexeme class for parsing binary data coming from Rserve"""
@@ -57,6 +68,7 @@ class Lexeme(list):
         self.hasAttr = hasAttr
         self.lexpos = lexpos
         self.attrLexeme = None
+        self.data = None
 
     def setAttr(self, attrLexeme):
         self.attrLexeme = attrLexeme
@@ -75,7 +87,7 @@ class Lexeme(list):
 
     @property
     def dataLength(self):
-        'Return length (in bytes) of actual REXPR data body'
+        """Return length (in bytes) of actual REXPR data body"""
         if self.hasAttr:
             if not self.attrLexeme:
                 raise RuntimeError('Attribute lexeme not yet set')
@@ -89,10 +101,6 @@ class Lexeme(list):
                (hex(self.rTypeCode), self.length, self.hasAttr, self.lexpos)
 
 
-class EndOfDataError(RserveError):
-    pass
-
-
 class Lexer(object):
     """Rserve message lexer
     Can either read a OOBMessage or a R Object
@@ -118,6 +126,15 @@ class Lexer(object):
             self._read = self.fp.recv
         else:
             self._read = self.fp.read
+        # The following attributes will be set thru 'readHeader()':
+        self.lexpos = None
+        self.messageSize = None
+        self.errCode = None
+        self.responseCode = None
+        self.responseOK = None
+        self.isOOB = False
+        self.oobType = None
+        self.oobUserCode = None
 
     def readHeader(self):
         """
@@ -135,9 +152,13 @@ class Lexer(object):
         self.lexpos = 0
 
         command = Command(struct.unpack('<I', self.read(4))[0])
-        self.messageSize = self.__unpack(XT_INT)
+        # Obtain lower 32bit part of message length:
+        messageSize1 = self.__unpack(XT_INT)
         dataOffset = self.__unpack(XT_INT)
-        messageSize2 = self.__unpack(XT_INT) #TODO: add to message size
+        assert dataOffset == 0, 'dataOffset > 0 is not implemented'
+        # Obtain upper 32bit part of message length:
+        messageSize2 = self.__unpack(XT_INT) << 32  # shift 32bits to the left
+        self.messageSize = messageSize2 + messageSize1
 
         self.isOOB = command.isOOB
         if self.isOOB:
@@ -148,7 +169,7 @@ class Lexer(object):
 
             if DEBUG:
                 print('oob type: %x, oob user code: %x, message size: %d' %
-                    (self.oobType, self.oobUserCode, self.messageSize))
+                      (self.oobType, self.oobUserCode, self.messageSize))
         else:
             self.errCode = command.errCode
 
@@ -160,13 +181,13 @@ class Lexer(object):
             else:
                 self.clearSocketData()
                 raise ValueError('Received illegal response code (%x)' %
-                                self.responseCode)
+                                 self.responseCode)
 
             if DEBUG:
                 print('response ok? %s (responseCode=%x), error-code: %x, '
-                    'message size: %d' %
-                    (self.responseOK, self.responseCode,
-                    self.errCode, self.messageSize))
+                      'message size: %d' %
+                      (self.responseOK, self.responseCode,
+                       self.errCode, self.messageSize))
 
         return self.messageSize
 
@@ -298,7 +319,7 @@ class Lexer(object):
     def xt_array_numeric(self, lexeme):
         raw = self.read(lexeme.dataLength)
         # TODO: swapping...
-        data = numpy.fromstring(raw, dtype=numpyMap[lexeme.rTypeCode])
+        data = numpy.frombuffer(raw, dtype=numpyMap[lexeme.rTypeCode])
         return data
 
     @fmap(XT_ARRAY_BOOL)
@@ -317,8 +338,19 @@ class Lexer(object):
         numBools = self.__unpack(XT_INT, 1)[0]
         # read the actual boolean values, including padding bytes:
         raw = self.read(lexeme.dataLength - 4)
-        data = numpy.fromstring(raw[:numBools],
-                                dtype=numpyMap[lexeme.rTypeCode])
+        # Check if the array contains any NA values (encoded as \x02).
+        # If so we need to convert the 2's to None's and use a numpy
+        # array of type Object otherwise numpy will cast the None's into False's.
+        # This is handled for us for numeric types since numpy can use it's own
+        # nan type, but here we need to help it out.
+        if 2 in raw:
+            data = numpy.frombuffer(raw[:numBools], dtype=numpy.int8).astype(object)
+            data[data == 2] = None
+        else:
+            data = numpy.frombuffer(
+                raw[:numBools],
+                dtype=numpyMap[lexeme.rTypeCode]
+            )
         return data
 
     @fmap(XT_ARRAY_STR)
@@ -336,8 +368,8 @@ class Lexer(object):
         strList = [stringEncode(byteString) for byteString in bytesStrList]
         return numpy.array(strList)
 
-    @fmap(XT_STR, XT_SYMNAME)
-    def xt_symname(self, lexeme):
+    @fmap(XT_STR)
+    def xt_str(self, lexeme):
         """
         A null-terminated string.
         It's length can be larger than the actual string since it is always a
@@ -348,6 +380,14 @@ class Lexer(object):
         byteStr = raw.split(b'\0', 1)[0]
         return stringEncode(byteStr)
 
+    @fmap(XT_SYMNAME)
+    def xt_symname(self, lexeme):
+        """
+        Just like a string, but in S4 classes, a special value for NULL exists
+        """
+        string = self.xt_str(lexeme)
+        return None if string == '\x01NULL\x01' else string
+
     @fmap(XT_NULL)
     def xt_null(self, lexeme):
         return None
@@ -376,6 +416,7 @@ class RParser(object):
         """
         self.lexer = Lexer(src)
         self.atomicArray = atomicArray
+        self.indentLevel = None
 
     def __getitem__(self, key):
         return self.parserMap[key]
@@ -394,11 +435,11 @@ class RParser(object):
 
     def _debugLog(self, lexeme, isRexpr=True):
         if DEBUG:
-            l = lexeme
+            lx = lexeme
             typeCodeDict = XTs if isRexpr else DTs
             print('%s %s (%s), hasAttr=%s, lexpos=%d, length=%s' %
-                  (self.__ind, typeCodeDict[l.rTypeCode], hex(l.rTypeCode),
-                   l.hasAttr, l.lexpos, l.length))
+                  (self.__ind, typeCodeDict[lx.rTypeCode], hex(lx.rTypeCode),
+                   lx.hasAttr, lx.lexpos, lx.length))
 
     def parse(self):
         """
@@ -412,7 +453,7 @@ class RParser(object):
         if self.lexer.messageSize > 0:
             try:
                 message = self._parse()
-            except:
+            except Exception:
                 # If any error is raised during lexing and parsing, make sure
                 # that the entire data is read from the input source if it is
                 # a socket, otherwise following attempts to
@@ -424,13 +465,14 @@ class RParser(object):
                 rserve_err_msg = ERRORS[self.lexer.errCode]
             except KeyError:
                 raise REvalError("R evaluation error (code=%d)" %
-                                self.lexer.errCode)
+                                 self.lexer.errCode)
             else:
                 raise RResponseError('Response error %s (error code=%d)' %
-                                    (rserve_err_msg, self.lexer.errCode))
+                                     (rserve_err_msg, self.lexer.errCode))
 
         if self.lexer.isOOB:
-            return OOBMessage(self.lexer.oobType, self.lexer.oobUserCode, message, self.lexer.messageSize)
+            return OOBMessage(self.lexer.oobType, self.lexer.oobUserCode,
+                              message, self.lexer.messageSize)
         else:
             return message
 
@@ -438,8 +480,8 @@ class RParser(object):
         dataLexeme = self.lexer.nextExprHdr()
         self._debugLog(dataLexeme, isRexpr=False)
         if dataLexeme.rTypeCode == DT_SEXP:
-            expression = self._parseExpr()
-            return self._postprocessData(expression.data)
+            lexeme = self._parseExpr()
+            return self._postprocessData(lexeme.data)
         else:
             raise NotImplementedError()
 
@@ -486,27 +528,27 @@ class RParser(object):
                 # For convenience reasons type-convert it into a native
                 # Python data type:
                 data = data[0]
-                if isinstance(data, (numpy.float, numpy.float64)):
+                if isinstance(data, (float, numpy.float64)):
                     # convert into native python float:
                     data = float(data)
-                elif isinstance(data, (numpy.int, numpy.int32, numpy.int64)):
+                elif isinstance(data, (int, numpy.int32, numpy.int64)):
                     # convert into native int or long, depending on value:
                     data = int(data)
-                elif isinstance(data, (numpy.complex, numpy.complex64,
+                elif isinstance(data, (complex, numpy.complex64,
                                        numpy.complex128)):
                     # convert into native python complex number:
                     data = complex(data)
-                elif isinstance(data, (numpy.string_, numpy.str_)):
+                elif isinstance(data, (numpy.string_, str)):
                     # convert into native python string:
                     data = str(data)
-                elif isinstance(data, (numpy.bool_, numpy.bool, numpy.bool8)):
+                elif isinstance(data, (bool, numpy.bool_)):
                     # convert into native python string
                     data = bool(data)
         return data
 
     @fmap(None)
     def xt_(self, lexeme):
-        'apply this for atomic data'
+        # apply this for atomic data
         return self._nextExprData(lexeme)
 
     @fmap(XT_ARRAY_BOOL, XT_ARRAY_INT, XT_ARRAY_DOUBLE, XT_ARRAY_STR)
@@ -559,7 +601,7 @@ class RParser(object):
             data.append(self._postprocessData(self._parseExpr().data))
 
         if lexeme.hasAttr and lexeme.attrTypeCode == XT_LIST_TAG:
-            # The vector is actually a tagge list, i.e. a list which allows
+            # The vector is actually a tagged list, i.e. a list which allows
             # to access its items by name (like in a dictionary). However items
             # are ordered, and there is not necessarily a name available for
             # every item.
@@ -598,6 +640,15 @@ class RParser(object):
         # Closure instance.
         return Closure(lexeme, aList1, aList2)
 
+    @fmap(XT_S4)
+    def xt_s4(self, lexeme):
+        """A S4 object only contains attributes, no other payload"""
+        if lexeme.hasAttr and lexeme.attrTypeCode == XT_LIST_TAG:
+            return S4(lexeme.attr)
+        else:
+            return S4([])
+
+
 ##############################################################################
 
 
@@ -620,3 +671,19 @@ class Closure(object):
 
     def __repr__(self):
         return '<Closure instance %d>' % id(self)
+
+
+class S4(dict):
+    """Very simple representation of a S4 instance"""
+    def __init__(self, source=(), **entries):
+        super(S4, self).__init__(source, **entries)
+
+        if 'class' in self:
+            self.classes = self['class']
+            del self['class']
+        else:
+            self.classes = []
+
+    def __repr__(self):
+        attrs = super(S4, self).__repr__()
+        return "<S4 classes={} {}>".format(self.classes, attrs)
