@@ -11,28 +11,33 @@ MIN_INT32 = -MAX_INT32
 
 # Rserve constants and mappings ###############################################
 
-RHEADER_SIZE = 16             # Rserve header size
+# Main Rserve header size [bytes]
+RHEADER_SIZE = 16
+
+# Header sizes (in SEXPR) without and with XT_LARGE or DT_LARGE flag [bytes]
+SMALL_DATA_HEADER_SIZE = 4
+LARGE_DATA_HEADER_SIZE = 8
 
 
 CMD_RESP = 0x10000            # all responses have this flag set
 
 RESP_OK  = CMD_RESP | 0x0001  # command succeeded; returned parameters depend
-                              #   on the command issued
+                              # on the command issued
 RESP_ERR = CMD_RESP | 0x0002  # command failed, check stats code
 
 
 CMD_OOB         = 0x20000     # out-of-band data - i.e. unsolicited messages
 
-OOB_SEND        = CMD_OOB | 0x1000 # OOB send - unsolicited SEXP sent from the
-                                   # R instance to the client. 12 LSB are
-                                   # reserved for application-specific code
-OOB_MSG         = CMD_OOB | 0x2000 # OOB message - unsolicited message sent
-                                   # from the R instance to the client
-                                   # requiring a response. 12 LSB are reserved
-                                   # for application-specific code
-OOB_STREAM_READ = CMD_OOB | 0x4000 # OOB stream read request - server requests
-                                   # streaming data from the client (typically
-                                   # streaming input for computation)
+OOB_SEND        = CMD_OOB | 0x1000  # OOB send - unsolicited SEXP sent from the
+                                    # R instance to the client. 12 LSB are
+                                    # reserved for application-specific code
+OOB_MSG         = CMD_OOB | 0x2000  # OOB message - unsolicited message sent
+                                    # from the R instance to the client
+                                    # requiring a response. 12 LSB are reserved
+                                    # for application-specific code
+OOB_STREAM_READ = CMD_OOB | 0x4000  # OOB stream read request - server requests
+                                    # streaming data from the client (typically
+                                    # streaming input for computation)
 
 ###############################################################################
 # Error codes
@@ -251,20 +256,20 @@ DTs = dict([(rTypeCode, dt_name) for (dt_name, rTypeCode) in locals().items()
             if dt_name.startswith('DT_')])
 
 
-BOOL_TRUE   = 1
-BOOL_FALSE  = 0
-BOOL_NA     = 2
+BOOL_TRUE = 1
+BOOL_FALSE = 0
+BOOL_NA = 2
 
 VALID_R_TYPES = [
     DT_SEXP, XT_BOOL, XT_INT, XT_DOUBLE, XT_STR, XT_SYMNAME, XT_VECTOR,
     XT_LIST_TAG, XT_LANG_TAG, XT_LIST_NOTAG, XT_LANG_NOTAG, XT_CLOS,
     XT_ARRAY_BOOL, XT_ARRAY_INT, XT_ARRAY_DOUBLE, XT_ARRAY_CPLX, XT_ARRAY_STR,
-    XT_VECTOR_EXP, XT_NULL, XT_UNKNOWN, XT_RAW
+    XT_VECTOR_EXP, XT_NULL, XT_UNKNOWN, XT_RAW, XT_S4
 ]
 
 STRING_TYPES = [str, numpy.string_, numpy.str_]
 if not PY3:
-    STRING_TYPES.append(unicode)
+    STRING_TYPES.append(unicode)  # noqa: F821      'unicode' unknown in Python3
 
 ###############################################################################
 # Mapping btw. numpy and R data types, in both directions
@@ -272,8 +277,7 @@ if not PY3:
 # map r-types and some python types to typecodes used in the 'struct' module
 structMap = {
     XT_BOOL:          'b',
-    numpy.bool:       'b',
-    numpy.bool_:      'b',
+    bool:             'b',
     XT_BYTE:          'B',
     XT_INT:           'i',
     int:              'i',
@@ -284,9 +288,9 @@ structMap = {
     float:            'd',
     numpy.double:     'd',
     complex:          'd',
-    numpy.complex:    'd',
+    complex:          'd',
     numpy.complex128: 'd',
-    }
+}
 
 # mapping to determine overall type of message.
 DT_Map = {
@@ -298,24 +302,23 @@ DT_Map = {
 
 numpyMap = {
     XT_ARRAY_BOOL:     numpy.bool_,
-    #   XT_BYTE:           numpy.byte,
     XT_ARRAY_INT:      numpy.int32,
     XT_ARRAY_DOUBLE:   numpy.double,     # double float64
-    XT_ARRAY_CPLX:     numpy.complex,
+    XT_ARRAY_CPLX:     complex,
     XT_ARRAY_STR:      numpy.string_,
-    }
+}
 
 # also add the inverse mapping to it:
 for k, v in list(numpyMap.items()):
     numpyMap[v] = k
 
 # some manual additions for numpy variants:
-numpyMap[numpy.complex128] = XT_ARRAY_CPLX
-numpyMap[numpy.int32]      = XT_ARRAY_INT
-numpyMap[numpy.int64]      = XT_ARRAY_INT
-numpyMap[numpy.long]       = XT_ARRAY_INT
-numpyMap[numpy.str_]       = XT_ARRAY_STR
-numpyMap[numpy.unicode_]   = XT_ARRAY_STR
+numpyMap[numpy.complex128]  = XT_ARRAY_CPLX
+numpyMap[numpy.int32]       = XT_ARRAY_INT
+numpyMap[numpy.int64]       = XT_ARRAY_INT
+numpyMap[numpy.compat.long] = XT_ARRAY_INT
+numpyMap[numpy.str_]        = XT_ARRAY_STR
+numpyMap[numpy.unicode_]    = XT_ARRAY_STR
 
 
 atom2ArrMap = {
@@ -325,26 +328,10 @@ atom2ArrMap = {
     float:             XT_ARRAY_DOUBLE,
     numpy.double:      XT_ARRAY_DOUBLE,
     complex:           XT_ARRAY_CPLX,
-    numpy.complex:     XT_ARRAY_CPLX,
     numpy.complex128:  XT_ARRAY_CPLX,
     str:               XT_ARRAY_STR,
     numpy.str_:        XT_ARRAY_STR,
     numpy.string_:     XT_ARRAY_STR,
     numpy.unicode_:    XT_ARRAY_STR,
     bool:              XT_ARRAY_BOOL,
-    numpy.bool:        XT_ARRAY_BOOL,
-    numpy.bool_:       XT_ARRAY_BOOL,
-    }
-
-
-###############################################################################
-# Some custom exceptions used in the pyRserve package
-
-
-class RserveError(Exception):
-    # base class for all errors for this package
-    pass
-
-
-class RParserError(RserveError):
-    pass
+}
